import 'package:cloud_firestore/cloud_firestore.dart';

abstract class IdItemInterface {
  String? id;
}

abstract class ReadOnlyDatabaseItem<T> extends IdItemInterface {
  Timestamp? lastModified;
  String getTypeName();
}

abstract class UpdateableDatabaseItem<T> extends ReadOnlyDatabaseItem<T> {
  T copy();
  Map<String, dynamic> toMap();

  bool equals(UpdateableDatabaseItem<T>? item) {
    if (item == null) return false;

    if (this is HasDatabaseSubItems) if (!(this as HasDatabaseSubItems)
        .equalSubItems(item as HasDatabaseSubItems)) return false;

    var map = toMap();
    var itemMap = item.toMap();

    if (map.length != itemMap.length) return false;

    for (var key in map.keys) {
      if (!itemMap.containsKey(key)) return false;
      if (key == "last_modified") continue;
      if ((map[key]?.toString() ?? "")
              .compareTo(itemMap[key]?.toString() ?? "") !=
          0) return false;
    }

    return true;
  }
}

abstract class HasDatabaseSubItems<T extends UpdateableDatabaseItem<T>> {
  List<T>? get subItems;
  List<DocumentReference> get subItemReferences;
  String get subItemTypeName;

  bool equalSubItems(HasDatabaseSubItems<T>? item) {
    if (item == null) return false;

    if (subItems != null || item.subItems != null) {
      if (subItems == null || item.subItems == null) return false;
      if (subItems!.length != item.subItems!.length) return false;
      for (var i = 0; i < subItems!.length; i++) {
        if (!subItems![i].equals(item.subItems![i])) return false;
      }
    }

    return true;
  }
}

extension DocumentSnapshotReferenceList on DocumentSnapshot {
  List<DocumentReference> getReferenceList(String fieldName) =>
      this.get(fieldName) == null
          ? <DocumentReference>[]
          : List.from(this.get(fieldName));
}
