import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

import 'navigator_widget.dart';

class MenuWidget extends StatefulWidget {
  final List<String> tabTitles;
  final List<IconData> tabIcons;
  final bool isExpandable;
  final Widget? header;
  final Widget? footer;
  final double headerHoverPadding;

  final Widget Function(BuildContext context, int pageIndex) pageBuilder;

  const MenuWidget({
    Key? key,
    required this.tabTitles,
    required this.tabIcons,
    required this.pageBuilder,
    this.isExpandable = true,
    this.header,
    this.headerHoverPadding = 35.0,
    this.footer,
  })  : assert(tabIcons.length == tabTitles.length),
        super(key: key);

  @override
  _MenuWidgetState createState() => _MenuWidgetState();
}

class _MenuWidgetState extends State<MenuWidget> {
  bool hover = false;
  int selectedTapPageIndex = 0;

  @override
  void initState() {
    super.initState();
    Future.delayed(
        Duration(milliseconds: 100),
        () =>
            SplitNavigator.of(SplitNavigatorSide.left)!.push(MaterialPageRoute(
              builder: (context) =>
                  widget.pageBuilder(context, selectedTapPageIndex),
            )));
  }

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      onExit:
          widget.isExpandable ? (event) => setState(() => hover = false) : null,
      onEnter:
          widget.isExpandable ? (event) => setState(() => hover = true) : null,
      child: AnimatedContainer(
        duration: Duration(milliseconds: 40),
        width: hover ? 200 : 50,
        color: CleanAndroidDesignDefaultColors.defaultColor,
        child: Stack(
          children: <Widget>[
            Column(
              children: <Widget>[
                AnimatedContainer(
                  duration: Duration(milliseconds: 40),
                  padding: EdgeInsets.symmetric(
                      vertical: 5.0,
                      horizontal: hover ? widget.headerHoverPadding : 5.0),
                  child: widget.header ?? Container(),
                ),
              ]..addAll(widget.tabTitles
                  .asMap()
                  .map((index, title) => MapEntry(
                      index,
                      Padding(
                        padding: const EdgeInsets.only(top: 8),
                        child:
                            _getMenuItem(widget.tabIcons[index], title, index),
                      )))
                  .values
                  .toList()),
            ),
            Align(
              alignment: Alignment.bottomLeft,
              child: widget.footer ?? Container(),
            )
          ],
        ),
      ),
    );
  }

  Widget _getMenuItem(IconData icon, String title, int index) {
    return Container(
      width: 200,
      child: Material(
        type: MaterialType.transparency,
        child: InkWell(
          hoverColor: Color.fromRGBO(80, 80, 80, 1),
          highlightColor: Color.fromRGBO(80, 80, 80, 1),
          onTap: () {
            _clickMenuItem(index);
          },
          child: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 14, vertical: 7),
            child: Row(
              children: <Widget>[
                Icon(
                  icon,
                  color: CleanAndroidDesignDefaultColors.defaultColorLight,
                  size: 23,
                ),
                Container(width: 14),
                Text(
                  title,
                  style: TextStyles.textStyleSize(16).copyWith(
                    color: CleanAndroidDesignDefaultColors.defaultColorLight,
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  void _clickMenuItem(int index) {
    if (LockedNavigatorWidget.isLocked()) {
      LockedNavigatorWidget.animate();
      return;
    }

    setState(() {
      selectedTapPageIndex = index;
    });

    SplitNavigator.of(SplitNavigatorSide.left)!
        .pushReplacement(MaterialPageRoute(
      builder: (context) => widget.pageBuilder(context, selectedTapPageIndex),
    ));
    SplitNavigator.of(SplitNavigatorSide.right)!
        .popUntil((route) => route is NoPageRoute);
  }
}
