import 'dart:async';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class TabScaffoldWidget extends StatefulWidget {
  final List<String> tabTitles;

  final Widget Function(BuildContext context, int? pageIndex) pageBuilder;

  const TabScaffoldWidget({
    Key? key,
    required this.tabTitles,
    required this.pageBuilder,
  }) : super(key: key);

  @override
  TabScaffoldWidgetState createState() => TabScaffoldWidgetState();
}

class TabScaffoldWidgetState extends State<TabScaffoldWidget> {
  int? selectedTapPageIndex;
  bool? _showTabBar;
  List<Widget>? _menuButtons;

  @override
  void initState() {
    super.initState();

    selectedTapPageIndex = 0;
    _showTabBar = true;
  }

  @override
  Widget build(BuildContext context) {
    context = context;
    SplitNavigator.mainContext = context;

    return Column(
      children: <Widget>[
        Expanded(
          child: widget.pageBuilder(context, selectedTapPageIndex),
        ),
        _getTabBar(context),
      ],
    );
  }

  Widget _getTabBar(BuildContext context) {
    return Container(
      color: CleanAndroidDesignDefaultColors.background,
      height: 60,
      width: MediaQuery.of(context).size.width,
      child: Row(
        children: _menuButtons ??
            widget.tabTitles
                .asMap()
                .map((index, title) =>
                    MapEntry(index, _getTabBarItem(title, index)))
                .values
                .toList(),
      ),
    );
  }

  Widget _getTabBarItem(String name, int index) {
    var enabled = index == selectedTapPageIndex;
    var textStyle = TextStyle(
      color:
          enabled ? CleanAndroidDesignDefaultColors.accent : Colors.grey[600],
      fontWeight: enabled ? FontWeight.w500 : FontWeight.w400,
      fontSize: 15,
      fontFamily: 'Roboto',
      decoration: TextDecoration.none,
    );
    var textWidth = _textSize(name, textStyle).width;
    return Expanded(
      child: Container(
        height: 35,
        child: Material(
          borderRadius: BorderRadius.circular(18),
          color: CleanAndroidDesignDefaultColors.background,
          child: InkWell(
            borderRadius: BorderRadius.circular(18),
            onTap: () {
              if (index == selectedTapPageIndex) return;
              setState(() {
                selectedTapPageIndex = index;
              });
            },
            child: Column(
              children: <Widget>[
                Container(
                  height: 4,
                ),
                Text(
                  name,
                  style: textStyle,
                ),
                Container(
                  width: textWidth,
                  height: 4,
                  child: enabled
                      ? LayoutBuilder(
                          builder: (BuildContext context,
                              BoxConstraints constraints) {
                            final dashWidth = 3.0;
                            final dashHeight = 3.0;
                            final dashCount =
                                (textWidth / (2 * dashWidth)).floor();
                            return Flex(
                              children: List.generate(dashCount, (_) {
                                return SizedBox(
                                  width: dashWidth,
                                  height: dashHeight,
                                  child: DecoratedBox(
                                    decoration: BoxDecoration(
                                        color: CleanAndroidDesignDefaultColors
                                            .accent,
                                        borderRadius: BorderRadius.circular(2)),
                                  ),
                                );
                              }),
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              direction: Axis.horizontal,
                            );
                          },
                        )
                      : Container(),
                )
              ],
              mainAxisAlignment: MainAxisAlignment.center,
            ),
          ),
        ),
      ),
    );
  }

  Size _textSize(String text, TextStyle style) {
    final TextPainter textPainter = TextPainter(
        text: TextSpan(text: text, style: style),
        maxLines: 1,
        textDirection: TextDirection.ltr)
      ..layout(minWidth: 0, maxWidth: double.infinity);
    return textPainter.size;
  }

  void showTheTabBar(bool show) {
    if (_showTabBar == show) return;

    Future.delayed(
        Duration(milliseconds: 0),
        () => setState(() {
              _showTabBar = show;
              print(_showTabBar);
            }));
  }

  void showMenuButtons(List<Widget>? menuButtons) {
    if ((_menuButtons == null && menuButtons == null) ||
        (_menuButtons != null &&
            menuButtons != null &&
            _menuButtons!.length == menuButtons.length &&
            _menuButtons!
                .every((_w) => menuButtons.any((w) => w.key == _w.key))))
      return;

    Future.delayed(
        Duration(milliseconds: 0),
        () => setState(() {
              _menuButtons = menuButtons;
            }));
  }
}
