import 'dart:async';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class LoadingWidget extends StatelessWidget {
  final Color color;
  final Color? indicatorColor;
  final bool canPop;

  static StreamController<String?> _messageController =
      new StreamController<String?>.broadcast();

  static set showMessage(String? value) => _messageController.add(value);

  const LoadingWidget({
    Key? key,
    this.color = const Color.fromARGB(100, 255, 255, 255),
    this.indicatorColor,
    this.canPop = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => canPop,
      child: Container(
        width: double.infinity,
        height: double.infinity,
        color: color,
        child: Stack(
          children: [
            Align(
              alignment: Alignment.center,
              child: Container(
                width: 70,
                height: 70,
                child: CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color?>(indicatorColor ??
                      CleanAndroidDesignDefaultColors.defaultColor),
                ),
              ),
            ),
            Align(
              alignment: Alignment.center,
              child: Padding(
                padding: EdgeInsets.only(top: 150),
                child: StreamBuilder<String?>(
                  stream: _messageController.stream,
                  builder: (context, snapshot) => AnimatedSwitcher(
                    duration: Duration(milliseconds: 300),
                    child: (snapshot.hasData && snapshot.data != null)
                        ? Text(
                            snapshot.data!,
                            style: TextStyles.textStyleSize(16).copyWith(
                              color:
                                  CleanAndroidDesignDefaultColors.defaultColor,
                            ),
                            key: Key(snapshot.data!),
                          )
                        : Container(),
                  ),
                ),
              ),
            )
          ],
        ),
      ),
    );
  }

  static show(BuildContext context, {bool canPop = false}) {
    LoadingWidget.showMessage = null;
    Navigator.push(
        context,
        TransparentRoute(
          builder: (context) => LoadingWidget(canPop: canPop),
        ));
  }
}

class TransparentRoute extends PageRoute<void> {
  TransparentRoute({
    required this.builder,
    RouteSettings? settings,
  }) : super(settings: settings, fullscreenDialog: false);

  final WidgetBuilder builder;

  @override
  bool get opaque => false;

  @override
  Color? get barrierColor => null;

  @override
  String? get barrierLabel => null;

  @override
  bool get maintainState => true;

  @override
  Duration get transitionDuration => Duration(milliseconds: 350);

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    final result = builder(context);
    return FadeTransition(
      opacity: Tween<double>(begin: 0, end: 1).animate(animation),
      child: Semantics(
        scopesRoute: true,
        explicitChildNodes: true,
        child: result,
      ),
    );
  }
}
